@@ -21,32 +21,32 @@ setOldClass("recipe")
 #' @seealso \code{\link{resample}}
 #' 
 MLControl <- function(surv_times = numeric(), seed = NULL, ...) {
-  MLControl_depwarn(...)
+  MLControl_stop(...)
   if (is.null(seed)) seed <- sample.int(.Machine$integer.max, 1)
   new("MLControl", surv_times = surv_times, seed = seed)
 }
 
 
-MLControl_depwarn <- function(summary = NULL, cutoff = NULL,
-                              cutoff_index = NULL, na.rm = NULL, ...) {
+MLControl_stop <- function(summary = NULL, cutoff = NULL,
+                           cutoff_index = NULL, na.rm = NULL, ...) {
   if (!is.null(summary)) {
-    depwarn("'summary' argument to MLControl is deprecated",
-            "apply the 'performance' function to Resamples output directly")
+    stop("'summary' argument to MLControl is deprecated;\n",
+         "apply the 'performance' function to Resamples output directly")
   }
   
   if (!is.null(cutoff)) {
-    depwarn("'cutoff' argument to MLContorl is deprecated",
-            "specify in calls to 'performance' instead")
+    stop("'cutoff' argument to MLContorl is deprecated;\n",
+         "specify in calls to 'performance' instead")
   }
   
   if (!is.null(cutoff_index)) {
-    depwarn("'cutoff_index' argument to MLControl is deprecated",
-            "specify in calls to 'performance' instead")
+    stop("'cutoff_index' argument to MLControl is deprecated;\n",
+         "specify in calls to 'roc_index' instead")
   }
   
   if (!is.null(na.rm)) {
-    depwarn("'na.rm' argument to MLControl is deprecated",
-            "specify in calls to 'performance' instead")
+    stop("'na.rm' argument to MLControl is deprecated;\n",
+         "specify in calls to 'performance' instead")
   }
 }
 
@@ -243,10 +243,15 @@ setClass("MLMetric",
 #' \code{"matrix"}, \code{"numeric"}, \code{"ordered"}, and \code{"Surv"}.
 #' @param params list of user-specified model parameters to be passed to the
 #' \code{fit} function.
-#' @param nvars function to determine the number of variables included in the
-#' model from the \code{data} frame used by \code{fit}.
+#' @param grid tuning grid function whose first agument \code{x} is a
+#' \code{\link{ModelFrame}} of the model fit data and formula, followed by a
+#' \code{length} to use in generating sequences of parameter values, a number of
+#' grid points to sample at \code{random}, and an ellipsis (\code{...}).
+#' @param design character string indicating whether the type of design matrix
+#' used to fit the model is a \code{"\link{model.matrix}"}, a data.frame
+#' of the original predictor variable \code{"terms"}, or unknown (default).
 #' @param fit model fitting function whose arguments are a \code{formula}, a
-#' \code{data} frame, case \code{weights}, and an ellipsis (\code{...}).
+#' \code{data} frame, case \code{weights}, and an ellipsis.
 #' @param predict model prediction function whose arguments are the
 #' \code{object} returned by \code{fit}, a \code{newdata} frame of predictor
 #' variables, optional vector of \code{times} at which to predict survival,
@@ -254,8 +259,13 @@ setClass("MLMetric",
 #' @param varimp variable importance function whose arguments are the
 #' \code{object} returned by \code{fit}, optional arguments passed from calls
 #' to \code{\link{varimp}}, and an ellipsis.
+#' @param ... arguments passed from other methods.
 #' 
 #' @details
+#' If supplied, the \code{grid} function should return a list whose elements are
+#' named after and contain values of parameters to include in a tuning grid to
+#' be constructed automatically by the package.
+#' 
 #' Values returned by the \code{predict} functions should be formatted according
 #' to the response variable types below.
 #' \describe{
@@ -265,7 +275,7 @@ setClass("MLMetric",
 #' \item{matrix}{a matrix of predicted responses.}
 #' \item{numeric}{a vector or column matrix of predicted responses.}
 #' \item{Surv}{a matrix whose columns contain survival probabilities at
-#' \code{times} if supplied or a vector of survival predictions otherwise.}
+#' \code{times} if supplied or a vector of predicted survival means otherwise.}
 #' }
 #' 
 #' The \code{varimp} function should return a vector of importance values named
@@ -297,25 +307,29 @@ setClass("MLMetric",
 #' res <- resample(type ~ ., data = Pima.tr, model = LogisticModel)
 #' summary(res)
 #' 
-MLModel <- function(name = "MLModel", label = name, packages = character(0),
-                    types = character(0), params = list(),
-                    nvars = function(data) NULL,
+MLModel <- function(name = "MLModel", label = name, packages = character(),
+                    types = character(), params = list(),
+                    grid = function(x, length, random, ...) NULL,
+                    design = c(NA, "model.matrix", "terms"),
                     fit = function(formula, data, weights, ...)
                       stop("no fit function"),
                     predict = function(object, newdata, times, ...)
                       stop("no predict function"),
-                    varimp = function(object, ...) NULL) {
+                    varimp = function(object, ...) NULL, ...) {
   
   stopifnot(types %in% c("binary", "factor", "matrix", "numeric", "ordered",
                          "Surv"))
   
+  MLModel_depwarn(...)
+  
   new("MLModel",
       name = name,
       label = label,
       packages = packages,
       types = types,
       params = params,
-      nvars = nvars,
+      grid = grid,
+      design = match.arg(design),
       fit = fit,
       fitbits = MLFitBits(packages = packages,
                           predict = predict,
@@ -323,13 +337,22 @@ MLModel <- function(name = "MLModel", label = name, packages = character(0),
 }
 
 
+MLModel_depwarn <- function(nvars = NULL, ...) {
+  if (!is.null(nvars)) {
+    depwarn("'nvars' argument to MLModel is deprecated",
+            "specify the design matrix type with 'design' instead")
+  }
+}
+
+
 setClass("MLModel",
   slots = c(name = "character",
             label = "character",
             packages = "character",
             types = "character",
             params = "list",
-            nvars = "function",
+            grid = "function",
+            design = "character",
             fit = "function",
             fitbits = "MLFitBits")
 )
@@ -413,9 +436,9 @@ Confusion <- function(...) {
 }
 
 
-ConfusionMatrix <- function(object) {
-   structure(object, class = c("ConfusionMatrix", "table"))
-}
+ConfusionMatrix <- setClass("ConfusionMatrix",
+  contains = c("table", "matrix")
+)
 
 
 HTestPerformanceDiff <- setClass("HTestPerformanceDiff",
@@ -485,7 +508,7 @@ PerformanceDiff <- setClass("PerformanceDiff",
 
 
 MLModelTune <- setClass("MLModelTune",
-  slots = c(grid = "data.frame",
+  slots = c(tune_grid = "data.frame",
             performance = "Performance",
             selected = "numeric"),
   contains = "MLModel"
