@@ -23,7 +23,11 @@
 #' @details
 #' \describe{
 #' \item{Response Types:}{\code{factor}}
+#' \item{\link[=tune]{Automatic Tuning} Grid Parameters:}{
+#'   \code{mfinal}, \code{maxdepth}, \code{coeflearn}*
 #' }
+#' }
+#' * included only in randomly sampled grid points
 #' 
 #' Further model details can be found in the source link below.
 #' 
@@ -55,7 +59,15 @@ AdaBoostModel <- function(boos = TRUE, mfinal = 100,
     packages = "adabag",
     types = "factor",
     params = params,
-    nvars = function(data) nvars(data, design = "terms"),
+    grid = function(x, length, random, ...) {
+      params <- list(
+        mfinal = round(seq_range(0, 25, c(1, 200), length + 1)),
+        maxdepth = 1:min(length, 30)
+      )
+      if (random) params$coeflearn <- c("Breiman", "Freund", "Zhu")
+      params
+    },
+    design = "terms",
     fit = function(formula, data, weights, ...) {
       assert_equal_weights(weights)
       data <- model.frame(formula, data)
