@@ -23,7 +23,14 @@
 #' @details
 #' \describe{
 #' \item{Response Types:}{\code{factor}}
+#' \item{\link[=tune]{Automatic Tuning} Grid Parameters}{
+#' \itemize{
+#'   \item FDAModel: \code{nprune}, \code{degree}*
+#'   \item PDAModel: \code{lambda}
 #' }
+#' }
+#' }
+#' * included only in randomly sampled grid points
 #' 
 #' The \code{\link{predict}} function for this model additionally accepts the
 #' following argument.
@@ -52,7 +59,16 @@ FDAModel <- function(theta = NULL, dimension = NULL, eps = .Machine$double.eps,
     packages = "mda",
     types = "factor",
     params = params(environment()),
-    nvars = function(data) nvars(data, design = "model.matrix"),
+    grid = function(x, length, random, ...) {
+      modelfit <- fit(x, model = EarthModel(pmethod = "none"))
+      max_terms <- min(2 + 0.75 * nrow(modelfit$dirs), 200)
+      params <- list(
+        nprune = round(seq(2, max_terms, length = length))
+      )
+      if (random) params$degree <- 1:2
+      params
+    },
+    design = "model.matrix",
     fit = function(formula, data, weights, ...) {
       mda::fda(formula, data = data, weights = weights, ...)
     },
@@ -79,5 +95,10 @@ PDAModel <- function(lambda = 1, df = NULL, ...) {
   model <- do.call(FDAModel, args, quote = TRUE)
   model@name <- "PDAModel"
   model@label <- "Penalized Discriminant Analysis"
+  model@grid <- function(x, length, ...) {
+    list(
+      lambda = c(0, 10^seq_inner(-5, 1, length - 1))
+    )
+  }
   model
 }
