@@ -54,7 +54,14 @@ setGeneric("convert_response",
            function(object, x, ...) standardGeneric("convert_response"))
 
 
-setMethod("convert_response", c("ANY", "ANY"), function(object, x, ...) x)
+setMethod("convert_response", c("ANY", "ANY"),
+  function(object, x, ...) x
+)
+
+
+setMethod("convert_response", c("factor", "factor"),
+  function(object, x, ...) x
+)
 
 
 setMethod("convert_response", c("factor", "matrix"),
@@ -80,6 +87,13 @@ setMethod("convert_response", c("integer", "numeric"),
 )
 
 
+setMethod("convert_response", c("ordered", "matrix"),
+  function(object, x, ...) {
+    ordered(max.col(x), levels = 1:nlevels(object), labels = levels(object))
+  }
+)
+
+
 setMethod("convert_response", c("Surv", "matrix"),
   function(object, x, cutoff = 0.5, ...) {
     x <- x <= cutoff
