@@ -27,7 +27,7 @@ metricinfo <- function(...) {
   if (length(args) == 0) args <- as.list(.metric_names)
   info <- do.call(.metricinfo, args)
   
-  is_type <- !sapply(info, is, class2 = "list")
+  is_type <- !mapply(is, info, "list")
   if (any(is_type)) {
     info_metrics <- if (all(is_type)) metricinfo() else info[!is_type]
     info_types <- do.call(.metricinfo_types, info[is_type])
