@@ -96,7 +96,7 @@ plot.MLModelTune <- function(x, metrics = NULL, stat = mean,
   perf <- x@performance
   type <- match.arg(type)
   if (type == "line") {
-    grid <- x@grid
+    grid <- x@tune_grid
     if (any(dim(grid) == 0)) stop("no tuning parameters to plot")
     stats <- apply(perf, c(3, 2), function(x) stat(na.omit(x))) %>%
       as.data.frame.table
@@ -115,8 +115,9 @@ plot.MLModelTune <- function(x, metrics = NULL, stat = mean,
     }
     df$metric <- factor(df$metric, metrics)
     
-    mapping <- if (ncol(grid) > 1) {
-      df$group <- do.call(interaction, grid[-1])
+    indices <- sapply(grid[-1], function(x) length(unique(x)) > 1)
+    mapping <- if (any(indices)) {
+      df$group <- interaction(grid[-1][indices])
       aes(x, y, color = group, shape = group)
     } else {
       aes(x, y)
@@ -124,8 +125,8 @@ plot.MLModelTune <- function(x, metrics = NULL, stat = mean,
     ggplot(df, mapping) +
       geom_line() +
       geom_point() +
-      labs(x = names(grid)[1], y = "Values", color = "Params Group",
-           shape = "Params Group") +
+      labs(x = names(grid)[1], y = "Values", color = "Group",
+           shape = "Group") +
       facet_wrap(~ metric, scales = "free")
   } else {
     plot(perf, metrics = metrics, stat = stat, type = type, ...)
