@@ -2,12 +2,12 @@
 #' 
 #' Calculate measures of the relative importance of predictors in a model.
 #' 
-#' @param object MLModelFit object from a model fit.
+#' @param object \code{MLModelFit} object from a model fit.
 #' @param scale logical indicating whether importance measures should be scaled
 #' to range from 0 to 100.
 #' @param ... arguments passed to model-specific variable importance functions.
 #' 
-#' @return VarImp class object.
+#' @return \code{VarImp} class object.
 #' 
 #' @seealso \code{\link{fit}}, \code{\link{plot}}
 #'
@@ -24,13 +24,18 @@
 varimp <- function(object, scale = TRUE, ...) {
   stopifnot(is(object, "MLModelFit"))
   requireModelNamespaces(fitbit(object, "packages"))
-  vi <- fitbit(object, "varimp")(object, ...)
-  if (is.null(vi)) vi <- undef_varimp(object)
+  vi <- fitbit(object, "varimp")(unMLModelFit(object), ...)
+  if (is.null(vi)) vi <- varimp_undef(object)
   VarImp(as(vi, "VarImp"), scale = scale)
 }
 
 
-undef_varimp <- function(object) {
+varimp_pchisq <- function(object) {
+  pchisq(coef(object)^2 / diag(vcov(object)), 1)
+}
+
+
+varimp_undef <- function(object) {
   warn("variable importance not defined for ", class(object)[1])
   varnames <- labels(terms(fitbit(object, "x")))
   structure(rep(NA_integer_, length(varnames)), names = varnames)
